(*
    LCS - A study for a simple differencing program
    Properties window
    Copyright (c) 2002-2003 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit FormProperties;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls;

type
  TFileProps = record
    Name: string;
    Location: string;
    Size: integer;
    Attr: integer;
  end;

type
  TPropertiesForm = class(TForm)
    BtnCancel: TButton;
    BtnOK: TButton;
    TabControl: TTabControl;
    LblLocation: TLabel;
    LblName: TLabel;
    EditFileName: TEdit;
    EditFileLocation: TEdit;
    BtnGoToLocation: TButton;
    LblSize: TLabel;
    LblFileSize: TLabel;
    LblAttributes: TLabel;
    ChkReadonly: TCheckBox;
    ChkHidden: TCheckBox;
    ChkArchive: TCheckBox;
    BtnApply: TButton;
    procedure TabControlChange(Sender: TObject);
    procedure BtnGoToLocationClick(Sender: TObject);
    procedure AttributeChanged(Sender: TObject);
    procedure BtnApplyClick(Sender: TObject);
  private
    { Private declarations }
    FName: array[0..1] of string;
    FProps: array[0..1] of TFileProps;
    FAttrs: array[0..1] of integer;
    function AttrsFromCheckBoxes: integer;
    procedure AssignProperties( const Props: TFileProps );
  public
    { Public declarations }
    procedure SetFiles( const F1, F2: string );
  end;

var
  PropertiesForm: TPropertiesForm;

implementation

uses ShellApi;

{$R *.DFM}

procedure GetFileProperties( const FileName: string; var Props: TFileProps );
var
  F: TSearchRec;
begin
  Props.Name := ExtractFileName( FileName );
  Props.Location := ExtractFilePath( FileName );
  Props.Attr := 0;
  Props.Size := 0;
  if( FindFirst( FileName, faAnyFile, F ) = 0 ) then begin
    Props.Attr := F.Attr;
    Props.Size := F.Size;
    FindClose( F );
  end;
end;

procedure TPropertiesForm.SetFiles( const F1, F2: string );
begin
  FName[0] := F1;
  FName[1] := F2;
  GetFileProperties( F1, FProps[0] );
  GetFileProperties( F2, FProps[1] );
  AssignProperties( FProps[0] );
  FAttrs[0] := FProps[0].Attr;
  FAttrs[1] := FProps[1].Attr;
  BtnApply.Enabled := false;
end;

procedure TPropertiesForm.TabControlChange(Sender: TObject);
begin
  AssignProperties( FProps[TabControl.TabIndex] )
end;

function FileSizeToString( Size: integer ): string;
begin
  if( Size < 1024 ) then begin
    Result := IntToStr(Size) + ' bytes';
  end
  else begin
    if( Size < 1024*1024 ) then begin
      Result := Format( '%.1f K', [Size / 1024] );
    end
    else begin
      Result := Format( '%.2f M', [Size / (1024*1024)] );
    end;
    Result := Result + Format( ' (%.0n bytes)', [Size*1.0] );
  end;
end;

procedure TPropertiesForm.AssignProperties( const Props: TFileProps );
begin
  EditFileName.Text := Props.Name;
  EditFileLocation.Text := Props.Location;
  ChkReadonly.Checked := (Props.Attr and faReadOnly) <> 0;
  ChkHidden.Checked := (Props.Attr and faHidden) <> 0;
  ChkArchive.Checked := (Props.Attr and faArchive) <> 0;
  LblFileSize.Caption := FileSizeToString( Props.Size );
end;

procedure TPropertiesForm.BtnGoToLocationClick(Sender: TObject);
begin
  ShellExecute( 0, nil, PChar(EditFileLocation.Text), nil, nil, SW_NORMAL );
end;

function TPropertiesForm.AttrsFromCheckBoxes: integer;
begin
  Result := FProps[ TabControl.TabIndex ].Attr;
  Result := Result and not (faReadOnly or faHidden or faArchive);
  if( ChkReadonly.Checked ) then Result := Result or faReadOnly;
  if( ChkHidden.Checked ) then Result := Result or faHidden;
  if( ChkArchive.Checked ) then Result := Result or faArchive;
end;

procedure TPropertiesForm.AttributeChanged(Sender: TObject);
begin
  FAttrs[ TabControl.TabIndex ] := AttrsFromCheckBoxes;
  BtnApply.Enabled := (FAttrs[0] <> FProps[0].Attr) or (FAttrs[1] <> FProps[1].Attr );
end;

procedure TPropertiesForm.BtnApplyClick(Sender: TObject);
var
  I: integer;
begin
  for I:=0 to 1 do begin
    if( FAttrs[I] <> FProps[I].Attr ) then begin
      if( FileSetAttr(FName[I], FAttrs[I]) = 0 ) then begin
        FProps[I].Attr := FAttrs[I];
      end
      else begin
        MessageDlg( 'Cannot change the attributes of file ' + FName[I],
            mtError, [mbOk], 0 );
      end;
    end
  end;
  AttributeChanged(Self);
end;

end.
